<?php

namespace Teknisa\MAN\Services\AppsIntegrations\Apps;

use Teknisa\MAN\Factories\OMClient\ClientTypes;

use Teknisa\MAN\Factories\OMClient\OMClientRepo as OMClientRepo;

use Teknisa\MAN\Factories\Retail\ParamGeral     as ParamGeralRepo;
use Teknisa\MAN\Factories\Retail\Integracao     as IntegracaoRepo;

abstract class AAppService implements IAppService {
    private const UNABLE_TO_CALL_OM_API     = "Não foi possível se comunicar com o serviço do OM para obter os dados do cliente.";
    private const FAILED_TO_UPDATE_JOB_NAME = "Não foi possível gerar o nome do JOB para esta integração.";

    private const SAAS_ID     = "SAAS";
    private const HOSTING_ID  = "HOSTING";
    
    protected $paramGeralRepo;
    protected $integracaoRepo;
    protected $OMRepo;
    protected $environment;
    
    public function __construct(
            ParamGeralRepo $paramGeralRepo, 
            IntegracaoRepo $integracaoRepo,
            OMClientRepo $OMRepo, 
            $environment) {
        $this->paramGeralRepo = $paramGeralRepo;
        $this->integracaoRepo = $integracaoRepo;
        $this->OMRepo = $OMRepo;
        $this->environment = $environment;
        
        $this->setOMInfo();
    }
    
    /**
     * $clientType -> One of the Teknisa\MAN\Factories\OMClient\ClientTypes constant.
     * 
     * returns the respective client id to the API.
     */
    protected function getClientId($clientType) {
        if ($clientType === ClientTypes::SAAS_TYPE) {
            return self::SAAS_ID;
        } elseif ($clientType === ClientTypes::HOSTING_TYPE) {
            return self::HOSTING_ID;
        } else {
            throw new \Exception('Tipo de cliente desconhecido.');
        }
    }
    
    protected function getAPIURL() {
        return $this->paramGeralRepo->getAppsJenkinsAPIURL();
    }
    
    public function getAPIToken() {
        return $this->paramGeralRepo->getAppsJenkinsAPIToken();
    }
    
    public function getJobName($idIntegracao) {
        $jobName = $this->integracaoRepo->getJobName($idIntegracao);
        
        if(!$jobName) {
            // Gets a new standard job name.
            $jobName = $this->getNewJobName($idIntegracao);
            
            // Saves the new Job Name into the DB.
            if(!$this->integracaoRepo->updateJobName($idIntegracao, $jobName)) {
                throw new \Exception(self::FAILED_TO_UPDATE_JOB_NAME);
            }
        }
        
        return $jobName;
    }
    
    private function getNewJobName($idIntegracao) {
        $omData = $this->omData;
        return $omData['NRORG'] . '_' . $idIntegracao . '_' . $omData['NMFANTORG'];
    }

    private function setOMInfo() {
        try {
            set_time_limit(60); // One minute to call OM API...
            $this->omData = $this->OMRepo->getOrganizationByNRORG($this->environment->getNrOrgTrab());
        } catch(\Exception $e) {
            throw new \Exception(self::UNABLE_TO_CALL_OM_API);    
        }
    }
    
    protected function getOMInfo() {
        return $this->omData;
    }
}